@@ -1,5 +1,7 @@
 L.Editable = L.Class.extend({
 
+    includes: [L.Mixin.Events],
+
     statics: {
         FORWARD: 1,
         BACKWARD: -1
@@ -17,14 +19,22 @@ L.Editable = L.Class.extend({
         L.setOptions(this, options);
         this.map = map;
         this.editLayer = this.createEditLayer();
+        this.featuresLayer = this.createFeaturesLayer();
         this.newClickHandler = this.createNewClickHandler();
         this.forwardLineGuide = this.createLineGuide();
         this.backwardLineGuide = this.createLineGuide();
+    },
 
+    fireAndForward: function (type, e) {
+        e = e || {};
+        e.editTools = this;
+        this.fire(type, e);
+        this.map.fire(type, e);
     },
 
     createLineGuide: function () {
-        return L.polyline([], {dashArray: '5,10', weight: 1});
+        var options = L.extend({dashArray: '5,10', weight: 1}, this.options.lineGuideOptions);
+        return L.polyline([], options);
     },
 
     createVertexIcon: function (options) {
@@ -42,6 +52,10 @@ L.Editable = L.Class.extend({
         return this.options.editLayer || new L.LayerGroup().addTo(this.map);
     },
 
+    createFeaturesLayer: function () {
+        return this.options.featuresLayer || new L.LayerGroup().addTo(this.map);
+    },
+
     moveForwardLineGuide: function (latlng) {
         if (this.forwardLineGuide._latlngs.length) {
             this.forwardLineGuide._latlngs[1] = latlng;
@@ -111,17 +125,23 @@ L.Editable = L.Class.extend({
         this.unregisterForDrawing();
     },
 
+    connectCreatedToMap: function (layer) {
+        return this.featuresLayer.addLayer(layer);
+    },
+
     startPolyline: function (latlng) {
-        var line = this.createPolyline([]).connectCreatedToMap(this.map),
-            editor = line.enableEdit();
+        var line = this.createPolyline([]);
+        this.connectCreatedToMap(line);
+        var editor = line.enableEdit();
         editor.startDrawingForward();
         if (latlng) editor.newPointForward(latlng);
         return line;
     },
 
     startPolygon: function (latlng) {
-        var polygon = this.createPolygon([]).connectCreatedToMap(this.map),
-            editor = polygon.enableEdit();
+        var polygon = this.createPolygon([]);
+        this.connectCreatedToMap(polygon);
+        var editor = polygon.enableEdit();
         editor.startDrawingForward();
         if (latlng) editor.newPointForward(latlng);
         return polygon;
@@ -129,8 +149,9 @@ L.Editable = L.Class.extend({
 
     startMarker: function (latlng) {
         latlng = latlng || this.map.getCenter();
-        var marker = this.createMarker(latlng).connectCreatedToMap(this.map),
-            editor = marker.enableEdit();
+        var marker = this.createMarker(latlng);
+        this.connectCreatedToMap(marker);
+        var editor = marker.enableEdit();
         editor.startDrawing();
         return marker;
     },
@@ -149,20 +170,20 @@ L.Editable = L.Class.extend({
     },
 
     createPolyline: function (latlngs) {
-        var line = new this.options.polylineClass(latlngs);
-        this.map.fire('editable:created', {layer: line});
+        var line = new this.options.polylineClass(latlngs, {editOptions: {editTools: this}});
+        this.fireAndForward('editable:created', {layer: line});
         return line;
     },
 
     createPolygon: function (latlngs) {
-        var polygon = new this.options.polygonClass(latlngs);
-        this.map.fire('editable:created', {layer: polygon});
+        var polygon = new this.options.polygonClass(latlngs, {editOptions: {editTools: this}});
+        this.fireAndForward('editable:created', {layer: polygon});
         return polygon;
     },
 
     createMarker: function (latlng) {
-        var marker = new this.options.markerClass(latlng);
-        this.map.fire('editable:created', {layer: marker});
+        var marker = new this.options.markerClass(latlng, {editOptions: {editTools: this}});
+        this.fireAndForward('editable:created', {layer: marker});
         return marker;
     }
 
@@ -216,14 +237,18 @@ L.Editable.VertexMarker = L.Marker.extend({
 
     onAdd: function (map) {
         L.Marker.prototype.onAdd.call(this, map);
-        L.DomEvent.on(this.dragging._draggable, 'drag', this.onDrag, this);
+        this.on('drag', this.onDrag);
+        this.on('dragstart', this.onDragStart);
+        this.on('dragend', this.onDragEnd);
         this.on('click', this.onClick);
         this.on('contextmenu', this.onContextMenu);
         this.on('mousedown touchstart', this.onMouseDown);
-        this.addMiddleMarkers();
+        if (!this.editor.tools.options.skipMiddleMarkers) this.addMiddleMarkers();
     },
 
     onDrag: function (e) {
+        e.vertex = this;
+        this.editor.onVertexMarkerDrag(e);
         var iconPos = L.DomUtil.getPosition(this._icon),
             latlng = this._map.layerPointToLatLng(iconPos);
         this.latlng.lat = latlng.lat;
@@ -238,6 +263,16 @@ L.Editable.VertexMarker = L.Marker.extend({
         }
     },
 
+    onDragStart: function (e) {
+        e.vertex = this;
+        this.editor.onVertexMarkerDragStart(e);
+    },
+
+    onDragEnd: function (e) {
+        e.vertex = this;
+        this.editor.onVertexMarkerDragEnd(e);
+    },
+
     onClick: function (e) {
         e.vertex = this;
         this.editor.onVertexMarkerClick(e);
@@ -398,7 +433,7 @@ L.Editable.BaseEditor = L.Class.extend({
         this.feature = feature;
         this.feature.editor = this;
         this.editLayer = new L.LayerGroup();
-        this.tools = this.options.tools || map.editTools;
+        this.tools = this.options.editTools || map.editTools;
     },
 
     enable: function () {
@@ -419,40 +454,40 @@ L.Editable.BaseEditor = L.Class.extend({
         return this;
     },
 
-    _fireAndForward: function (type, e) {
+    fireAndForward: function (type, e) {
         e = e || {};
         e.layer = this.feature;
         this.feature.fire(type, e);
         if (this.feature.multi) this.feature.multi.fire(type, e);
-        this.map.fire(type, e);
+        this.tools.fireAndForward(type, e);
     },
 
     onEnable: function () {
-        this._fireAndForward('editable:enable');
+        this.fireAndForward('editable:enable');
     },
 
     onDisable: function () {
-        this._fireAndForward('editable:disable');
+        this.fireAndForward('editable:disable');
     },
 
     onEditing: function () {
-        this._fireAndForward('editable:editing');
+        this.fireAndForward('editable:editing');
     },
 
     onStartDrawing: function () {
-        this._fireAndForward('editable:drawing:start');
+        this.fireAndForward('editable:drawing:start');
     },
 
     onEndDrawing: function () {
-        this._fireAndForward('editable:drawing:end');
+        this.fireAndForward('editable:drawing:end');
     },
 
     onCancelDrawing: function () {
-        this._fireAndForward('editable:drawing:cancel');
+        this.fireAndForward('editable:drawing:cancel');
     },
 
     onCommitDrawing: function () {
-        this._fireAndForward('editable:drawing:commit');
+        this.fireAndForward('editable:drawing:commit');
     },
 
     startDrawing: function () {
@@ -489,7 +524,7 @@ L.Editable.BaseEditor = L.Class.extend({
     },
 
     onNewClickHandlerClicked: function (e) {
-        this._fireAndForward('editable:drawing:click', e);
+        this.fireAndForward('editable:drawing:click', e);
     },
 
     isNewClickValid: function (latlng) {
@@ -584,7 +619,7 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
         return new this.tools.options.middleMarkerClass(left, right, latlngs, this);
     },
 
-    onVertexMarkerClick: function (e, vertex) {
+    onVertexMarkerClick: function (e) {
         var index = e.vertex.getIndex();
         if (e.originalEvent.ctrlKey) {
             this.onVertexMarkerCtrlClick(e);
@@ -614,31 +649,43 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
     },
 
     onVertexDeleted: function (e) {
-        this._fireAndForward('editable:vertex:deleted', e);
+        this.fireAndForward('editable:vertex:deleted', e);
     },
 
     onVertexMarkerCtrlClick: function (e) {
-        this._fireAndForward('editable:vertex:ctrlclick', e);
+        this.fireAndForward('editable:vertex:ctrlclick', e);
     },
 
     onVertexMarkerShiftClick: function (e) {
-        this._fireAndForward('editable:vertex:shiftclick', e);
+        this.fireAndForward('editable:vertex:shiftclick', e);
     },
 
     onVertexMarkerAltClick: function (e) {
-        this._fireAndForward('editable:vertex:altclick', e);
+        this.fireAndForward('editable:vertex:altclick', e);
     },
 
     onVertexMarkerContextMenu: function (e) {
-        this._fireAndForward('editable:vertex:contextmenu', e);
+        this.fireAndForward('editable:vertex:contextmenu', e);
     },
 
     onVertexMarkerMouseDown: function (e) {
-        this._fireAndForward('editable:vertex:mousedown', e);
+        this.fireAndForward('editable:vertex:mousedown', e);
     },
 
     onMiddleMarkerMouseDown: function (e) {
-        this._fireAndForward('editable:middlemarker:mousedown', e);
+        this.fireAndForward('editable:middlemarker:mousedown', e);
+    },
+
+    onVertexMarkerDrag: function (e) {
+        this.fireAndForward('editable:vertex:drag', e);
+    },
+
+    onVertexMarkerDragStart: function (e) {
+        this.fireAndForward('editable:vertex:dragstart', e);
+    },
+
+    onVertexMarkerDragEnd: function (e) {
+        this.fireAndForward('editable:vertex:dragend', e);
     },
 
     startDrawing: function () {
@@ -803,7 +850,7 @@ var EditableMixin = {
     createEditor: function (map) {
         map = map || this._map;
         var Klass = this.options.editorClass || this.getEditorClass(map);
-        return new Klass(map, this);
+        return new Klass(map, this, this.options.editOptions);
     },
 
     enableEdit: function () {
@@ -830,10 +877,6 @@ var EditableMixin = {
       } else {
         this.enableEdit();
       }
-    },
-
-    connectCreatedToMap: function (map) {
-        return this.addTo(map);
     }
 
 };
